package ru.csasq.cit_is_bot

import androidx.compose.animation.AnimatedVisibility
import androidx.compose.animation.expandHorizontally
import androidx.compose.animation.fadeIn
import androidx.compose.animation.fadeOut
import androidx.compose.animation.shrinkHorizontally
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.ExtendedFloatingActionButton
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.NavigationBar
import androidx.compose.material3.NavigationBarItem
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Snackbar
import androidx.compose.material3.SnackbarHost
import androidx.compose.material3.SnackbarHostState
import androidx.compose.material3.Text
import androidx.compose.material3.TopAppBar
import androidx.compose.runtime.*
import androidx.compose.ui.text.style.TextAlign
import androidx.navigation.NavDestination.Companion.hierarchy
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.currentBackStackEntryAsState
import androidx.navigation.compose.rememberNavController
import cit_is_bot.composeapp.generated.resources.Res
import cit_is_bot.composeapp.generated.resources.baseline_add_24
import cit_is_bot.composeapp.generated.resources.baseline_chat_24
import cit_is_bot.composeapp.generated.resources.baseline_dashboard_24
import cit_is_bot.composeapp.generated.resources.baseline_edit_calendar_24
import cit_is_bot.composeapp.generated.resources.baseline_supervisor_account_24
import org.jetbrains.compose.resources.DrawableResource
import org.jetbrains.compose.resources.painterResource
import ru.csasq.cit_is_bot.ui.theme.darkScheme

sealed class Screen(
    val route: String,
    val title: String,
    val icon: DrawableResource,
) {
    data object Dashboard : Screen(
        route = "dashboard",
        title = "Дашборд",
        icon = Res.drawable.baseline_dashboard_24,
    )
    data object Messages : Screen(
        route = "scenarios",
        title = "Сценарии",
        icon = Res.drawable.baseline_edit_calendar_24,
    )
    data object Chats : Screen(
        route = "chats",
        title = "Чаты",
        icon = Res.drawable.baseline_chat_24,
    )
    data object Users : Screen(
        route = "users",
        title = "Пользователи",
        icon = Res.drawable.baseline_supervisor_account_24,
    )
}

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun App() {
    val navController = rememberNavController()

    MaterialTheme(
        colorScheme = darkScheme,
    ) {
        val snackbarHostState = remember {
            SnackbarHostState()
        }
        val scheduleFloatingActionButtonVisibleState = remember {
            mutableStateOf(false)
        }
        val createRuleDialogOpeningState = remember {
            mutableStateOf(false)
        }
        Scaffold(
            topBar = {
                TopAppBar(
                    title = {
                        Text(
                            text = "Сообщения",
                        )
                    },
                )
            },
            bottomBar = {
                NavigationBar {
                    val navBackStackEntry = navController.currentBackStackEntryAsState()
                    val items = listOf(
                        Screen.Dashboard,
                        Screen.Messages,
                        Screen.Chats,
                        Screen.Users,
                    )
                    items.forEach { screen ->
                        NavigationBarItem(
                            icon = {
                                Icon(
                                    painter = painterResource(screen.icon),
                                    contentDescription = null,
                                )
                            },
                            label = {
                                Text(
                                    text = screen.title,
                                    textAlign = TextAlign.Center,
                                )
                            },
                            selected = navBackStackEntry.value?.destination?.hierarchy?.any {
                                it.route == screen.route
                            } == true,
                            onClick = {
                                navController.navigate(screen.route)
                            },
                        )
                    }
                }
            },
            snackbarHost = {
                SnackbarHost(
                    hostState = snackbarHostState,
                ) {
                    Snackbar(
                        snackbarData = it,
                        containerColor = MaterialTheme.colorScheme.errorContainer,
                        contentColor = MaterialTheme.colorScheme.error,
                        actionColor = MaterialTheme.colorScheme.onErrorContainer,
                    )
                }
            },
            floatingActionButton = {
                AnimatedVisibility(
                    visible = scheduleFloatingActionButtonVisibleState.value,
                    enter = fadeIn() + expandHorizontally(),
                    exit = fadeOut() + shrinkHorizontally()
                ) {
                    ExtendedFloatingActionButton(
                        text = {
                            Text(
                                text = "Создать правило",
                            )
                        },
                        icon = {
                            Icon(
                                painter = painterResource(Res.drawable.baseline_add_24),
                                contentDescription = null,
                            )
                        },
                        onClick = {
                            createRuleDialogOpeningState.value = true
                        },
                        expanded = true,
                    )
                }
            },
        ) {
            paddingValues -> NavHost(
                navController = navController,
                startDestination = Screen.Dashboard.route,
            ) {
                composable(Screen.Dashboard.route) {
                    scheduleFloatingActionButtonVisibleState.value = false
//                    CallProtectionHomeScreen(navController, paddingValues, dataStore, snackbarHostState)
                }
                composable(Screen.Messages.route) {
//                    CallProtectionScheduleScreen(navController, paddingValues, database)
                    scheduleFloatingActionButtonVisibleState.value = true

//                    when {
//                        createRuleDialogOpeningState.value -> CallProtectionRuleDialog(
//                            database = database,
//                            scheduleRule = null,
//                            state = createRuleDialogOpeningState,
//                        )
//                    }
                }
                composable(Screen.Chats.route) {
                    scheduleFloatingActionButtonVisibleState.value = false
//                    CallProtectionCallLogScreen(navController, paddingValues)
                }
                composable(Screen.Users.route) {
                    scheduleFloatingActionButtonVisibleState.value = false
//                    CallProtectionSettingsScreen(navController, paddingValues, dataStore)
                }
            }
        }
    }
}